#include "Group.h"

#include <cassert>
#include <sstream>
#include <string_view>  // string_view requires C++17 (either defined in CMakeLists.txt, or in Visual Studio, /Project properties -> Configuration properties->C / C++->Language->C++ Language standard / and select "ISO C++ 17 Standard"

#include "Circle.h"
#include "SymbolReference.h"

using namespace std;

Group::Group(const pugi::xml_node& node, Group* parent, Drawing *drawing) : Shape{node, parent} {
  for (auto child : node.children()) {
    assert(
        (child.name() == "Circle"sv) ||
        (child.name() == "Group"sv) ||
        (child.name() == "SymbolDefinition"sv) ||
        (child.name() == "SymbolReference"sv));
    // thanks to : using namespace std::literals;
    if (child.name() == "Circle"sv)
      children.push_back(std::make_unique<Circle>(child, this));
    else if (child.name() == "Group"sv)
      children.push_back(std::make_unique<Group>(child, this, drawing));
    else if (child.name() == "SymbolReference"sv)
      children.push_back(std::make_unique<SymbolReference>(child, this, drawing));
    // else it is "SymbolDefinition" which we just ignore
  }
}

Group::Group(std::string const& label, double x, double y, Group* parent)
    : Shape{label, x, y, parent} {}

void Group::add_child(unique_ptr<Shape> shape) {
  children.push_back(std::move(shape));
}

void Group::clone_with_new_parent(Group* parent) {
  auto g = std::make_unique<Group>(get_label_for_clone(), get_x(), get_y(), parent);
  for (auto const& child : children) {
    child->clone_with_new_parent(g.get());
  }
  if (parent != nullptr) {
    parent->add_child(std::move(g));
  }
}

void Group::draw(SFML_output* out, const double x_origin, const double y_origin) const {
  for (auto const& child : children) {
    child->draw(out, x_origin + get_x(), y_origin + get_y());
  }
}

std::string Group::dump(std::string const& indent) const {
  ostringstream oss;
  oss << indent << "Group " << Shape::dump(indent) << "children: [" << endl;
  for (auto const& c : children) {
    oss << c->dump(indent + "| ");
  }
  oss << indent << "]" << endl;
  return oss.str();
}

Shape* Group::find_shape_by_label(const string_view a_label) {
  if (Shape::find_shape_by_label(a_label) != nullptr) return this;
  for (auto const& c : children) {
    if (const auto result = c->find_shape_by_label(a_label); result != nullptr)
      return result;
  }
  return nullptr;
}

void Group::set_color(std::string const& aColor) {
  for (auto const& child : children) {
    child->set_color(aColor);
  }
}
